#define _LEGACY_HEADERS

#include <htc.h>
#include <pic16f886.h>
#include "hitec_c_i2c.h"
#include "lcd_sub.h"
#include "port_init.h"
#include "wait_time.h"
#include "max7219_sub.h"
#include "ad.h"

__CONFIG(	DEBUGDIS & LVPDIS & FCMDIS & IESODIS & BORDIS
			& DUNPROTECT & UNPROTECT & MCLREN & PWRTEN & WDTDIS & INTIO);
__CONFIG(	0xffff & BORV21);

#define		_XTAL_FREQ		4000000
#define		AKARUSA_MIN		30
#define		LOOP_MAX		50
#define		SP_INIT			500		

unsigned char	i2c_ondo_addr=0x48 << 1;
unsigned char	i2c_kiatu_addr=0x5c << 1;
unsigned char	i2c_rtc_addr=0x51 << 1;

unsigned char	data[4];

unsigned int	ondo_sub;
unsigned long	ondo_keisan;
unsigned char	ondo_hi;
unsigned char	ondo_low;
unsigned char	ondo_fugo;

unsigned char	kiatu_out_xl;
unsigned char	kiatu_out_l;
unsigned char	kiatu_out_h;
unsigned long	kiatu_keisan;

unsigned short	ad4_lev_in;
unsigned short	ad4_level;
unsigned char	situdo;

unsigned short	ad8_lev_in;
unsigned short	ad8_level;

unsigned char	sw1_key;
unsigned char	sw1_rep;
unsigned char	sw1_cnt;
unsigned char	sw2_key;
unsigned char	sw2_rep;
unsigned char	sw2_cnt;
unsigned char	sw3_key;
unsigned char	sw3_rep;
unsigned char	sw3_cnt;
unsigned char	sw4_key;
unsigned char	sw4_rep;
unsigned char	sw4_cnt;


unsigned char	sw1p;
unsigned char	sw2p;
unsigned char	sw3p;
unsigned char	sw4p;

unsigned char	loop;
unsigned char	loop_sub;
unsigned char	next;
unsigned char	mode;

unsigned char	sec;
unsigned char	min;
unsigned char	hour;
unsigned char	day;
unsigned char	mon;
unsigned char	year;
unsigned char	set_count;

/*---------------------ONDO-----------------------*/
void	ondo_init(void)
{
	data[0]=0x80;
	i2c_data_write(i2c_ondo_addr,0x03,1,data);
}

void	read_ondo(void)
{
	data[0]=0x80 | 0x20;
	i2c_data_write(i2c_ondo_addr,0x03,1,data);	
	__delay_ms(240);

	i2c_data_read(i2c_ondo_addr,0x00,1,data);
	ondo_hi=data[0];
	if((ondo_hi&0x80)==0x80){
		ondo_fugo=1;
	}
	else{
		ondo_fugo=0;
	}
	ondo_hi=ondo_hi & 0x7f;

	i2c_data_read(i2c_ondo_addr,0x01,1,data);
	ondo_low=data[0];

	ondo_sub=(unsigned int)ondo_hi*256+(unsigned int)ondo_low;

	ondo_keisan=((unsigned long)ondo_sub)*78;
	ondo_keisan=ondo_keisan/1000;

}

/*---------------------KIATU----------------------*/
void	kiatu_init(void)
{
	data[0]=0x90;
	i2c_data_write(i2c_kiatu_addr,0x20,1,data);
}
void	read_kiatu(void)
{
	data[0]=0x01;
	while(data[0]==0x00){
		i2c_data_read(i2c_kiatu_addr,0x21,1,data);
	}	
	i2c_data_read(i2c_kiatu_addr,0x28,1,data);
	kiatu_out_xl=data[0];
	i2c_data_read(i2c_kiatu_addr,0x29,1,data);
	kiatu_out_l=data[0];
	i2c_data_read(i2c_kiatu_addr,0x2a,1,data);
	kiatu_out_h=data[0];

	kiatu_keisan=(unsigned long)kiatu_out_h*256*256+(unsigned long)kiatu_out_l*256+(unsigned long)kiatu_out_xl;
	kiatu_keisan=kiatu_keisan*10;
	kiatu_keisan=kiatu_keisan/4096;


}
/*---------------------SHITUDO--------------------*/
void	read_situdo(void)
{
	ad_input_an4();
	ad4_level=ad4_lev_in;
	ad4_level=ad4_level*100/310;
	situdo=(unsigned char)ad4_level;
}
/*--------------------AKARUSA---------------------*/
void	read_akarusa(void)
{
	ad_input_an8();
	ad8_level=ad8_lev_in;
}
/*---------------------RTC------------------------*/
unsigned char	rtc_read(unsigned char	addr)
{
	i2c_data_read(i2c_rtc_addr,addr,1,data);

	return data[0];
}
void	rtc_set1(void)
{
	data[0]=sec;
	data[1]=min;
	data[2]=hour;
	data[3]=day;
	i2c_data_write(i2c_rtc_addr,0x02,4,data);
}
void	rtc_set2(void)
{
	data[0]=mon;
	data[1]=year;
	i2c_data_write(i2c_rtc_addr,0x07,2,data);
}
void	rtc_stop(void)
{
	data[0]=0x20;
	i2c_data_write(i2c_rtc_addr,0x00,1,data);
}
void	rtc_start(void)
{
	data[0]=0x00;
	i2c_data_write(i2c_rtc_addr,0x00,1,data);	
}
void	rtc_init(void)
{
	rtc_stop();
	data[0]=0x00;
	i2c_data_write(i2c_rtc_addr,0x01,1,data);

	sec=0;min=0;hour=0;day=1;
	rtc_set1();

	mon=1;year=0x13;
	rtc_set2();

	rtc_start();
}
void	rtc_read_time(void)
{
	i2c_data_read(i2c_rtc_addr,0x02,3,data);
	sec=data[0]&0x7f;min=data[1]&0x7f;hour=data[2]&0x3f;
}
void	rtc_read_date(void)
{
	i2c_data_read(i2c_rtc_addr,0x05,4,data);
	day=data[0]&0x3f;mon=data[2]&0x1f;year=data[3];
}
unsigned char	bcd_up_100(unsigned char data)
{
	if((data&0x0f)==0x09){
		data=data+0x07;
	}
	else{
		data++;
	}
	if(data>=0xa0){
		data=0;
	}
	return data;
}
unsigned char	bcd_up_12(unsigned char data)
{
	if((data&0x0f)==0x09){
		data=data+0x07;
	}
	else{
		data++;
	}
	if(data>=0x13){
		data=0x01;
	}
	return data;
}
unsigned char	bcd_up_31(unsigned char data)
{
	if((data&0x0f)==0x09){
		data=data+0x07;
	}
	else{
		data++;
	}
	if(data>=0x32){
		data=0x01;
	}
	return data;
}
unsigned char	bcd_up_24(unsigned char data)
{
	if((data&0x0f)==0x09){
		data=data+0x07;
	}
	else{
		data++;
	}
	if(data>=0x24){
		data=0x00;
	}
	return	data;
}
unsigned char	bcd_up_60(unsigned char data)
{
	if((data&0x0f)==0x09){
		data=data+0x07;
	}
	else{
		data++;
	}
	if(data>=0x60){
		data=0x00;
	}
	return data;
}
unsigned char	bcd_down_100(unsigned char data)
{
	if(data==0x00){
		data=0x99;
		return data;
	}
	if((data&0x0f)==0x00){
		data=data-0x07;
	}
	else{
		data--;
	}
	return data;
}
unsigned char	bcd_down_12(unsigned char data)
{
	if(data==0x01){
		data=0x12;
		return data;
	}
	if((data&0x0f)==0x00){
		data=data-0x07;
	}
	else{
		data--;
	}
	return data;
}
unsigned char	bcd_down_31(unsigned char data)
{
	if(data==0x01){
		data=0x31;
		return data;
	}
	if((data&0x0f)==0x00){
		data=data-0x07;
	}
	else{
		data--;
	}
	return data;
}
unsigned char	bcd_down_24(unsigned char data)
{
	if(data==0x00){
		data=0x23;
		return data;
	}
	if((data&0x0f)==0x00){
		data=data-0x07;
	}
	else{
		data--;
	}
	return data;
}
unsigned char	bcd_down_60(unsigned char data)
{
	if(data==0x00){
		data=0x59;
		return data;
	}
	if((data&0x0f)==0x00){
		data=data-0x07;
	}
	else{
		data--;
	}
	return data;
}
/*---------------------MAIN-----------------------*/
void	main(void)
{
unsigned char	i;

	port_init();
	max7219_port_init();

	__delay_ms(2000);

	I2CInit();	

	lcd_init();
	lcd_clear();
	mode=0;

	lcd_disp_mode(mode);

	ondo_init();
	kiatu_init();

	if((rtc_read(0x02) & 0x80)==0x80){
		rtc_init();
	}
	else{
		rtc_read_time();
		rtc_read_date();
	}

	max7219_data_out(0,0x0f,0x00);
	max7219_data_out(1,0x0f,0x00);

	for(i=1;i<=8;i++){
		max7219_data_out(0,i,0x00);
		max7219_data_out(1,i,0x00);
	}

	max7219_data_out(0,0x09,0x00);
	max7219_data_out(1,0x09,0x00);

	max7219_data_out(0,0x0a,0x04);
	max7219_data_out(1,0x0a,0x04);

	max7219_data_out(0,0x0b,0x07);
	max7219_data_out(1,0x0b,0x07);

	max7219_data_out(0,0x0c,0x01);
	max7219_data_out(1,0x0c,0x01);

	max7219_disp_mode(mode);

	loop=0;
	loop_sub=0;

	while(1){

		sw1p=RC0;NOP();
		sw2p=RC1;NOP();
		sw3p=RC2;NOP();
		sw4p=RC5;NOP();

		/* key scan 1 */
		if (sw1p==0){
				sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if (sw2p==0){
				sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if (sw3p==0){
				sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			sw3_cnt=0;
		}
		/*------------------*/
		if (sw4p==0){
				sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/

		wait_time(1000);

		sw1p=RC0;NOP();
		sw2p=RC1;NOP();
		sw3p=RC2;NOP();
		sw4p=RC5;NOP();

		/* key scan 2 */
		if ((sw1p==0)&&(sw1_key==0)){
			sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if ((sw2p==0)&&(sw2_key==0)){
			sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if ((sw3p==0)&&(sw3_key==0)){
			sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			sw3_cnt=0;
		}
		/*------------------*/
		if ((sw4p==0)&&(sw4_key==0)){
			sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/

		/* key action */
	
		if((sw1_key==0)&&(sw1_rep==0)){
			sw1_rep=1;		
			switch(mode){
				case 0:	mode=1;break;
				case 1:	mode=2;break;
				case 2:	mode=3;break;
				case 3:	mode=4;break;
				case 4:	mode=0;break;
				case 8:	sec=0;
						rtc_set1();
						rtc_set2();
						rtc_start();
						mode=0;
						break;
			}
			lcd_disp_mode(mode);
			max7219_disp_mode(mode);
			loop=0;loop_sub=0;
		}
		/*------------------*/
		if((sw2_key==0)&&(sw2_rep==0)){
			sw2_rep=1;
			if(mode==8){		
				switch(set_count){
					case 0:	year=bcd_down_100(year);
							lcd_xy(0,0);lcd_date();
							loop=1;loop_sub=0;
							break;
					case 1:	mon=bcd_down_12(mon);
							lcd_xy(0,0);lcd_date();
							loop=1;loop_sub=0;
							break;
					case 2:	day=bcd_down_31(day);
							lcd_xy(0,0);lcd_date();
							loop=1;loop_sub=0;
							break;
					case 3:	hour=bcd_down_24(hour);
							lcd_xy(2,1);lcd_time();
							loop=1;loop_sub=0;
							break;
					case 4:	min=bcd_down_60(min);
							lcd_xy(2,1);lcd_time();
							loop=1;loop_sub=0;
							break;
				}
			}
			else{

			}
		}
		/*------------------*/
		if((sw3_key==0)&&(sw3_rep==0)){
			sw3_rep=1;		
			if(mode==8){
				switch(set_count){
					case 0:	year=bcd_up_100(year);
							lcd_xy(0,0);lcd_date();
							loop=1;loop_sub=0;
							break;
					case 1:	mon=bcd_up_12(mon);
							lcd_xy(0,0);lcd_date();
							loop=1;loop_sub=0;
							break;
					case 2:	day=bcd_up_31(day);
							lcd_xy(0,0);lcd_date();
							loop=1;loop_sub=0;
							break;
					case 3:	hour=bcd_up_24(hour);
							lcd_xy(2,1);lcd_time();
							loop=1;loop_sub=0;
							break;
					case 4:	min=bcd_up_60(min);
							lcd_xy(2,1);lcd_time();
							loop=1;loop_sub=1;
							break;
				}
			}
			else{
				mode=4;
				lcd_disp_mode(mode);
				max7219_disp_mode(mode);
				loop=0;loop_sub=0;
			}
		}
		/*------------------*/
		if((sw4_key==0)&&(sw4_rep==0)){
			sw4_rep=1;		
			sw4_cnt=0;
			if(mode==8){
				switch(set_count){
					case 0:	set_count=1;next=0;loop=0;loop_sub=0;
							lcd_xy(0,0);
							lcd_date();
							lcd_xy(2,1);
							lcd_time();							
							break;
					case 1:	set_count=2;next=0;loop=0;loop_sub=0;
							lcd_xy(0,0);
							lcd_date();
							lcd_xy(2,1);
							lcd_time();								
							break;
					case 2:	set_count=3;next=0;loop=0;loop_sub=0;
							lcd_xy(0,0);
							lcd_date();
							lcd_xy(2,1);
							lcd_time();	
							break;
					case 3:	set_count=4;next=0;loop=0;loop_sub=0;
							lcd_xy(0,0);
							lcd_date();
							lcd_xy(2,1);
							lcd_time();	
							break;
					case 4:	set_count=0;
							sec=0;
							rtc_set1();
							rtc_set2();
							rtc_start();
							mode=0;
							lcd_disp_mode(mode);
							max7219_disp_mode(mode);
							loop=0;loop_sub=0;
							break;
				}
			}
		}
		/*------------------*/
		if((sw4_key==0)&&(sw4_rep==1)){
			sw4_cnt++;
			if(sw4_cnt>=200){
				sw4_cnt=0;
				mode=8;
				lcd_disp_mode(mode);
				lcd_xy(0,0);
				lcd_date();
				lcd_xy(2,1);
				lcd_time();
				set_count=0;
				rtc_stop();
				loop=0;loop_sub=0;
				next=0;
			}
		}
		/*------------------*/


		if(loop==0){
			if(loop_sub==0){
				switch(mode){
					case 0:
					case 1:
					case 2:
					case 4:
					case 3:	read_ondo();	
							lcd_xy(1,1);
							lcd_ondo(ondo_fugo,(unsigned int)ondo_keisan);
							max7219_ondo(1,ondo_fugo,(unsigned int)ondo_keisan);
							break;
				}
			}
			switch(mode){
				case 0:	if(loop_sub==0){
							read_kiatu();
							lcd_xy(0,0);
							lcd_kiatu((unsigned int)kiatu_keisan);
							max7219_kiatu(0,(unsigned int)kiatu_keisan);
						}			
						break;
				case 1:	if(loop_sub==0){
							read_situdo();
							if(situdo>99){
								situdo=99;
							}
							lcd_xy(2,0);
							lcd_2keta10(situdo);
							max7219_situdo(0,situdo);
						}		
						break;
				case 2:	rtc_read_time();
						lcd_xy(2,0);
						lcd_time();
						max7219_time(0);
						break;
				case 3:	rtc_read_date();
						lcd_xy(0,0);
						lcd_date();
						max7219_date(0);
						break;
				case 4:	rtc_read_date();
						rtc_read_time();
						lcd_xy(0,0);
						lcd_date_time();
						max7219_date_time(0);
						break;
				case 8:	switch(set_count){
							case 0:	if(next==0){
										lcd_xy(0,0);
										lcd_date();
									}
									else{
										lcd_xy(0,0);
										lcd_print("__");
									}
									break;			
							case 1:	if(next==0){
										lcd_xy(0,0);
										lcd_date();
									}
									else{
										lcd_xy(3,0);
										lcd_print("__");
									}
									break;		
							case 2:	if(next==0){
										lcd_xy(0,0);
										lcd_date();
									}
									else{
										lcd_xy(6,0);
										lcd_print("__");
									}
									break;
							case 3:	if(next==0){
										lcd_xy(2,1);
										lcd_time();
									}
									else{
										lcd_xy(2,1);
										lcd_print("__");
									}
									break;		
							case 4:	if(next==0){
										lcd_xy(2,1);
										lcd_time();
									}
									else{
										lcd_xy(5,1);
										lcd_print("__");
									}
									break;
						}
						next++;
						if(next>=2){
							next=0;
						}		
						break;
			}
			if(loop_sub==0){
				read_akarusa();
				if(ad8_level<AKARUSA_MIN){
					max7219_data_out(0,0x0a,0x01);
					max7219_data_out(1,0x0a,0x01);
				}
				else{
					max7219_data_out(0,0x0a,0x04);
					max7219_data_out(1,0x0a,0x04);
				}
			}				
		}

		loop++;
		if(loop>=LOOP_MAX){
			loop=0;loop_sub++;
			if(loop_sub>2){loop_sub=0;}
		}


	}

}